@@ -1,41 +1,45 @@
+/* vim: colorcolumn=80 ts=4 sw=4
+ */
 /*
  * text.c
  *
- * Copyright (C) 2002 Sun Microsystems, Inc.
+ * Copyright © 2002 Sun Microsystems, Inc.
+ * Copyright © 2021-2023 Logan Rathbone
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * Authors: Glynn Foster <glynn.foster@sun.com>
+ * Original Author: Glynn Foster <glynn.foster@sun.com>
  */
 
-#include "config.h"
 
 #include "util.h"
 #include "zenity.h"
+
 #include <gio/gio.h>
 
 #ifdef HAVE_WEBKITGTK
-#include <webkit2/webkit2.h>
+#include <webkit/webkit.h>
 #endif
 
+#include <config.h>
+
 static ZenityTextData *zen_text_data;
 
-static void zenity_text_dialog_response (
-	GtkWidget *widget, int response, gpointer data);
-static void zenity_text_toggle_button (GtkToggleButton *button, gpointer data);
+static void zenity_text_dialog_response (GtkWidget *widget, char *rstr, gpointer data);
+static void zenity_text_toggle_button (GtkCheckButton *button, AdwMessageDialog *dialog);
 
 #ifdef HAVE_WEBKITGTK
 static void
@@ -71,6 +75,7 @@ zenity_configure_webkit (WebKitWebView *web_view) {
 		FALSE,
 		NULL);
 	g_object_set (G_OBJECT (settings), "enable-page-cache", FALSE, NULL);
+	g_object_set (G_OBJECT (settings), "enable-plugins", FALSE, NULL);
 	/*
 	  Stick to defaults
 	  "enforce-96-dpi"           gboolean              : Read / Write /
@@ -144,53 +149,60 @@ zenity_text_webview_load_changed (
 	}
 }
 
-#endif
+#endif	/* HAVE_WEBKITGTK */
 
 static gboolean
-zenity_text_handle_stdin (
-	GIOChannel *channel, GIOCondition condition, gpointer data) {
+zenity_text_handle_stdin (GIOChannel *channel, GIOCondition condition,
+		gpointer data)
+{
+#define BUF_SIZE 1024
 	static GtkTextBuffer *buffer;
 	static GtkTextView *text_view;
-	gchar buf[1024];
+	char buf[BUF_SIZE];
 
 	gsize len;
 
 	text_view = GTK_TEXT_VIEW (data);
 	buffer = gtk_text_view_get_buffer (text_view);
 
-	if ((condition & G_IO_IN) || (condition & (G_IO_IN | G_IO_HUP))) {
-		GError *error = NULL;
-		gint status;
+	if ((condition & G_IO_IN) || (condition & (G_IO_IN | G_IO_HUP)))
+	{
+		g_autoptr(GError) error = NULL;
+		int status;
 
 		while (channel->is_readable != TRUE)
 			;
 
 		do {
-			status = g_io_channel_read_chars (channel, buf, 1024, &len, &error);
-
-			while (gtk_events_pending ())
-				gtk_main_iteration ();
+			status = g_io_channel_read_chars (channel, buf, BUF_SIZE,
+					&len, &error);
 
+			while (g_main_context_pending (NULL)) {
+				g_main_context_iteration (NULL, FALSE);
+			}
 		} while (status == G_IO_STATUS_AGAIN);
 
-		if (status != G_IO_STATUS_NORMAL) {
+		if (status != G_IO_STATUS_NORMAL)
+		{
 			if (error) {
-				g_warning ("zenity_text_handle_stdin () : %s", error->message);
-				g_error_free (error);
+				g_warning ("%s: %s",
+						__func__, error->message);
 				error = NULL;
 			}
 			return FALSE;
 		}
 
-		if (len > 0) {
+		if (len > 0)
+		{
 			GtkTextIter end;
-			gchar *utftext;
+			g_autofree char *utftext = NULL;
 			gsize localelen;
 			gsize utflen;
 
 			gtk_text_buffer_get_end_iter (buffer, &end);
 
-			if (!g_utf8_validate (buf, len, NULL)) {
+			if (! g_utf8_validate (buf, len, NULL))
+			{
 				utftext = g_convert_with_fallback (buf,
 					len,
 					"UTF-8",
@@ -200,41 +212,45 @@ zenity_text_handle_stdin (
 					&utflen,
 					NULL);
 				gtk_text_buffer_insert (buffer, &end, utftext, utflen);
-				g_free (utftext);
-			} else {
+			}
+			else
+			{
 				gtk_text_buffer_insert (buffer, &end, buf, len);
 			}
-			if (zen_text_data->auto_scroll) {
-				GtkTextMark *mark = NULL;
-				mark = gtk_text_buffer_get_insert (buffer);
+
+			if (zen_text_data->auto_scroll)
+			{
+				GtkTextMark *mark = gtk_text_buffer_get_insert (buffer);
+
 				if (mark != NULL)
-					gtk_text_view_scroll_to_mark (
-						text_view, mark, 0.0, FALSE, 0, 0);
+				{
+					gtk_text_view_scroll_to_mark (text_view, mark,
+							0.0, FALSE, 0, 0);
+				}
 			}
 		}
 	}
-
 	return TRUE;
 }
 
 static void
-zenity_text_fill_entries_from_stdin (GtkTextView *text_view) {
+zenity_text_fill_entries_from_stdin (GtkTextView *text_view)
+{
 	GIOChannel *channel;
 
 	channel = g_io_channel_unix_new (0);
 	g_io_channel_set_encoding (channel, "UTF-8", NULL);
 	g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, NULL);
-	g_io_add_watch (
-		channel, G_IO_IN | G_IO_HUP, zenity_text_handle_stdin, text_view);
+	g_io_add_watch (channel,
+			G_IO_IN | G_IO_HUP, zenity_text_handle_stdin, text_view);
 }
 
 void
-zenity_text (ZenityData *data, ZenityTextData *text_data) {
-	GtkBuilder *builder;
+zenity_text (ZenityData *data, ZenityTextData *text_data)
+{
+	g_autoptr(GtkBuilder) builder = NULL;
 	GtkWidget *dialog;
-	GtkWidget *ok_button;
 	GtkWidget *checkbox;
-	GtkWidget *cancel_button;
 
 	GObject *text_view;
 	GtkTextBuffer *text_buffer;
@@ -243,41 +259,31 @@ zenity_text (ZenityData *data, ZenityTextData *text_data) {
 	GtkWidget *web_kit;
 	GtkWidget *scrolled_window;
 	GtkTextIter start_iter, end_iter;
-	gchar *content;
+	g_autofree char *content = NULL;
 #endif
+
 	zen_text_data = text_data;
-	builder =
-		zenity_util_load_ui_file ("zenity_text_dialog", "textbuffer1", NULL);
+	builder = zenity_util_load_ui_file ("zenity_text_dialog", "zenity_text_box", NULL);
 
 	if (builder == NULL) {
 		data->exit_code = zenity_util_return_exit_code (ZENITY_ERROR);
 		return;
 	}
 
-	gtk_builder_connect_signals (builder, NULL);
+	dialog = GTK_WIDGET(gtk_builder_get_object (builder,
+				"zenity_text_dialog"));
 
-	dialog =
-		GTK_WIDGET (gtk_builder_get_object (builder, "zenity_text_dialog"));
+	checkbox = GTK_WIDGET(gtk_builder_get_object (builder,
+				"zenity_text_checkbox"));
 
-	ok_button = GTK_WIDGET (
-		gtk_builder_get_object (builder, "zenity_text_close_button"));
-	cancel_button = GTK_WIDGET (
-		gtk_builder_get_object (builder, "zenity_text_cancel_button"));
-	checkbox =
-		GTK_WIDGET (gtk_builder_get_object (builder, "zenity_text_checkbox"));
-
-	g_signal_connect (G_OBJECT (dialog),
-		"response",
-		G_CALLBACK (zenity_text_dialog_response),
-		data);
+	g_signal_connect (dialog, "response",
+		G_CALLBACK(zenity_text_dialog_response), data);
 
 	if (data->dialog_title)
-		gtk_window_set_title (GTK_WINDOW (dialog), data->dialog_title);
+		adw_message_dialog_set_heading (ADW_MESSAGE_DIALOG(dialog), data->dialog_title);;
 
-	zenity_util_set_window_icon (
-		dialog, data->window_icon, ZENITY_IMAGE_FULLPATH ("zenity-text.png"));
-
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+	gtk_window_set_icon_name (GTK_WINDOW(dialog),
+			"accessories-text-editor");
 
 	text_buffer = gtk_text_buffer_new (NULL);
 	text_view = gtk_builder_get_object (builder, "zenity_text_view");
@@ -287,13 +293,24 @@ zenity_text (ZenityData *data, ZenityTextData *text_data) {
 	if (text_data->no_wrap)
 		gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_NONE);
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-	if (text_data->font) {
-		PangoFontDescription *fontDesc =
-			pango_font_description_from_string (text_data->font);
-		gtk_widget_override_font (GTK_WIDGET (text_view), fontDesc);
+	if (text_data->font)
+	{
+		PangoFontDescription *desc;
+		GtkStyleContext *context;
+		GtkCssProvider *provider;
+		g_autofree char *css_str = NULL;
+
+		desc = pango_font_description_from_string (text_data->font);
+		css_str = zenity_util_pango_font_description_to_css (desc);
+			
+		provider = gtk_css_provider_new ();
+		gtk_css_provider_load_from_data (provider, css_str, -1);
+
+		context = gtk_widget_get_style_context (GTK_WIDGET(text_view));
+		gtk_style_context_add_provider (context,
+				GTK_STYLE_PROVIDER(provider),
+				GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 	}
-G_GNUC_END_IGNORE_DEPRECATIONS
 
 	if (text_data->uri)
 		zenity_util_fill_file_buffer (text_buffer, text_data->uri);
@@ -303,48 +320,51 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 	if (text_data->editable)
 		zen_text_data->buffer = text_buffer;
 
-	if (data->extra_label) {
-		gint i = 0;
-		while (data->extra_label[i] != NULL) {
-			gtk_dialog_add_button (
-				GTK_DIALOG (dialog), data->extra_label[i], i);
-			i++;
-		}
+	if (data->extra_label)
+	{
+		ZENITY_UTIL_ADD_EXTRA_LABELS (dialog)
 	}
 
 	if (data->ok_label) {
-		gtk_button_set_label (GTK_BUTTON (ok_button), data->ok_label);
+		ZENITY_UTIL_SETUP_OK_BUTTON_LABEL (dialog);
 	}
 
-	if (data->cancel_label) {
-		gtk_button_set_label (GTK_BUTTON (cancel_button), data->cancel_label);
+	if (data->cancel_label)
+	{
+		ZENITY_UTIL_SETUP_CANCEL_BUTTON_LABEL (dialog);
 	}
 
 	if (text_data->checkbox) {
-		gtk_widget_set_visible (GTK_WIDGET (checkbox), TRUE);
-		gtk_widget_set_sensitive (GTK_WIDGET (ok_button), FALSE);
-		gtk_button_set_label (GTK_BUTTON (checkbox), text_data->checkbox);
+		gtk_widget_set_visible (GTK_WIDGET(checkbox), TRUE);
+		gtk_check_button_set_label (GTK_CHECK_BUTTON(checkbox), text_data->checkbox);
+		adw_message_dialog_set_response_enabled (ADW_MESSAGE_DIALOG(dialog), "ok", FALSE);
 
-		g_signal_connect (G_OBJECT (checkbox),
-			"toggled",
-			G_CALLBACK (zenity_text_toggle_button),
-			ok_button);
+		g_signal_connect (checkbox, "toggled", G_CALLBACK(zenity_text_toggle_button), dialog);
 	}
 
 	if (data->width > -1 || data->height > -1)
-		gtk_window_set_default_size (
-			GTK_WINDOW (dialog), data->width, data->height);
-	else
+	{
+		gtk_window_set_default_size (GTK_WINDOW (dialog),
+				data->width, data->height);
+	}
+	else {
 		gtk_window_set_default_size (GTK_WINDOW (dialog), 300, 400);
+	}
 
 	if (data->modal)
 		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 
 #ifdef HAVE_WEBKITGTK
-	if (text_data->html) {
-		web_kit = g_object_new (WEBKIT_TYPE_WEB_VIEW,
-				"is-ephemeral", TRUE,
-				NULL);
+	if (text_data->html)
+	{
+		/* "ephemeral" == private browsing */
+		g_autoptr(WebKitWebContext) wk_context = webkit_web_context_new ();
+		g_autoptr(WebKitNetworkSession) wk_session = webkit_network_session_new_ephemeral ();
+
+		web_kit = WEBKIT_WEB_VIEW (g_object_new (WEBKIT_TYPE_WEB_VIEW,
+				"web-context", wk_context,
+				"network-session", wk_session,
+				NULL));
 		scrolled_window = GTK_WIDGET (
 			gtk_builder_get_object (builder, "zenity_text_scrolled_window"));
 
@@ -358,78 +378,74 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 			webkit_web_view_load_uri (
 				WEBKIT_WEB_VIEW (web_kit), text_data->url);
 		} else {
-			gchar *cwd;
-			gchar *dirname;
-			gchar *dirname_uri;
+			g_autofree char *cwd = NULL;
+			g_autofree char *dirname = NULL;
+			g_autofree char *dirname_uri = NULL;
 			dirname = text_data->uri ? g_path_get_dirname (text_data->uri)
 									 : g_strdup ("/");
 			cwd = g_get_current_dir ();
 			dirname_uri = g_strconcat ("file://", cwd, "/", dirname, "/", NULL);
-			g_free (cwd);
-			g_free (dirname);
 			gtk_text_buffer_get_start_iter (text_buffer, &start_iter);
 			gtk_text_buffer_get_end_iter (text_buffer, &end_iter);
 			content = gtk_text_buffer_get_text (
 				text_buffer, &start_iter, &end_iter, TRUE);
 			webkit_web_view_load_html (
 				WEBKIT_WEB_VIEW (web_kit), content, dirname_uri);
-			g_free (dirname_uri);
-			g_free (content);
 		}
 
-		// We don't want user to click on links and navigate to another page.
-		// So, when the page finishes loading, we take handle of the requests.
-
+		/* We don't want user to click on links and navigate to another page.
+		 * So, when the page finishes loading, we take handle of the requests.
+		 */
 		g_signal_connect (G_OBJECT (web_kit),
 			"load-changed",
 			G_CALLBACK (zenity_text_webview_load_changed),
 			NULL);
 
-		gtk_widget_destroy (GTK_WIDGET (text_view));
-		gtk_container_add (GTK_CONTAINER (scrolled_window), web_kit);
-		gtk_widget_show (GTK_WIDGET (web_kit));
+		gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW(scrolled_window), web_kit);
 	}
-#endif
+#endif /* HAVE_WEBKITGTK */
 
-	zenity_util_show_dialog (dialog, data->attach);
+	zenity_util_show_dialog (dialog);
 
-	g_object_unref (builder);
-
-	if (data->timeout_delay > 0) {
+	if (data->timeout_delay > 0)
+	{
 		g_timeout_add_seconds (data->timeout_delay,
 			(GSourceFunc) zenity_util_timeout_handle,
 			dialog);
 	}
-
-	gtk_main ();
+	zenity_util_gapp_main (GTK_WINDOW(dialog));
 }
 
 static void
-zenity_text_toggle_button (GtkToggleButton *button, gpointer data) {
-	GtkWidget *ok_button = (GtkWidget *) data;
-	gtk_widget_set_sensitive (
-		GTK_WIDGET (ok_button), gtk_toggle_button_get_active (button));
+zenity_text_toggle_button (GtkCheckButton *button, AdwMessageDialog *dialog)
+{
+	adw_message_dialog_set_response_enabled (dialog, "ok", gtk_check_button_get_active (button));
 }
 
 static void
-zenity_text_dialog_output (ZenityData *zen_data) {
-	if (zen_text_data->editable) {
+zenity_text_dialog_output (ZenityData *zen_data)
+{
+	if (zen_text_data->editable)
+	{
 		GtkTextIter start, end;
-		gchar *text;
+		g_autofree char *text = NULL;
+
 		gtk_text_buffer_get_bounds (zen_text_data->buffer, &start, &end);
-		text =
-			gtk_text_buffer_get_text (zen_text_data->buffer, &start, &end, 0);
+		text = gtk_text_buffer_get_text (zen_text_data->buffer,
+				&start, &end, 0);
 		g_print ("%s", text);
-		g_free (text);
 	}
 }
 
 static void
-zenity_text_dialog_response (GtkWidget *widget, int response, gpointer data) {
+zenity_text_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
+{
 	ZenityData *zen_data = data;
+	ZenityExitCode response = zenity_util_parse_dialog_response (rstr);
 
-	switch (response) {
-		case GTK_RESPONSE_CLOSE:
+	switch (response)
+	{
+		case ZENITY_OK:
 			zenity_text_dialog_output (zen_data);
 			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
 			break;
@@ -439,12 +455,22 @@ zenity_text_dialog_response (GtkWidget *widget, int response, gpointer data) {
 			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_TIMEOUT);
 			break;
 
+		case ZENITY_CANCEL:
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_CANCEL);
+			break;
+
+		case ZENITY_ESC:
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
+			break;
+			
 		default:
 			if (zen_data->extra_label &&
-				response < g_strv_length (zen_data->extra_label))
+				response < (int)g_strv_length (zen_data->extra_label))
+			{
 				printf ("%s\n", zen_data->extra_label[response]);
+			}
 			zenity_util_exit_code_with_data (ZENITY_ESC, zen_data);
 			break;
 	}
-	gtk_main_quit ();
+	zenity_util_gapp_quit (GTK_WINDOW(widget), zen_data);
 }
